Index: channels/chan_agent.c
===================================================================
--- channels/chan_agent.c	(revision 188742)
+++ channels/chan_agent.c	(working copy)
@@ -318,6 +318,7 @@
 static char *complete_agent_logoff_cmd(const char *line, const char *word, int pos, int state);
 static struct ast_channel* agent_get_base_channel(struct ast_channel *chan);
 static int agent_set_base_channel(struct ast_channel *chan, struct ast_channel *base);
+static int agent_logoff(const char *agent, int soft);
 
 /*! \brief Channel interface description for PBX integration */
 static const struct ast_channel_tech agent_tech = {
@@ -520,8 +521,11 @@
 	struct ast_frame *f = NULL;
 	static struct ast_frame answer_frame = { AST_FRAME_CONTROL, AST_CONTROL_ANSWER };
 	const char *status;
-	ast_mutex_lock(&p->lock); 
+	ast_mutex_lock(&p->lock);
 	CHECK_FORMATS(ast, p);
+	if (!p->start) {
+		p->start = time(NULL);
+	}
 	if (p->chan) {
 		ast_copy_flags(p->chan, ast, AST_FLAG_EXCEPTION);
 		p->chan->fdno = (ast->fdno == AST_AGENT_FD) ? AST_TIMING_FD : ast->fdno;
@@ -544,8 +548,6 @@
 						p->loginstart = 0;
 							ast_log(LOG_NOTICE, "Agent '%s' didn't answer/confirm within %d seconds (waited %d)\n", p->name, p->autologoff, howlong);
 						agent_logoff_maintenance(p, p->loginchan, logintime, ast->uniqueid, "Autologoff");
-						if (persistent_agents)
-							dump_agents();
 					}
 				}
 				status = pbx_builtin_getvar_helper(p->chan, "CHANLOCALSTATUS");
@@ -563,28 +565,37 @@
 			ast_devstate_changed(AST_DEVICE_UNAVAILABLE, "Agent/%s", p->agent);
 			p->acknowledged = 0;
 		}
- 	} else {
- 		/* if acknowledgement is not required, and the channel is up, we may have missed
- 		   an AST_CONTROL_ANSWER (if there was one), so mark the call acknowledged anyway */
- 		if (!p->ackcall && !p->acknowledged && p->chan && (p->chan->_state == AST_STATE_UP))
-  			p->acknowledged = 1;
- 		switch (f->frametype) {
- 		case AST_FRAME_CONTROL:
- 			if (f->subclass == AST_CONTROL_ANSWER) {
- 				if (p->ackcall) {
- 					ast_verb(3, "%s answered, waiting for '%c' to acknowledge\n", p->chan->name, p->acceptdtmf);
- 					/* Don't pass answer along */
- 					ast_frfree(f);
- 					f = &ast_null_frame;
- 				} else {
- 					p->acknowledged = 1;
- 					/* Use the builtin answer frame for the 
+	} else {
+		/* if acknowledgement is not required, and the channel is up, we may have missed
+			an AST_CONTROL_ANSWER (if there was one), so mark the call acknowledged anyway */
+		if (!p->ackcall && !p->acknowledged && p->chan && (p->chan->_state == AST_STATE_UP)) {
+			p->acknowledged = 1;
+		}
+
+		if (!p->acknowledged) {
+			int howlong = time(NULL)- p->start;
+			if (p->autologoff && (howlong > p->autologoff)) {
+				agent_logoff_maintenance(p, p->loginchan, (time(NULL) - p->loginstart), ast->uniqueid, "Autologoff");
+				agent_logoff(p->agent, 0);
+			}
+		}
+		switch (f->frametype) {
+		case AST_FRAME_CONTROL:
+			if (f->subclass == AST_CONTROL_ANSWER) {
+				if (p->ackcall) {
+					ast_verb(3, "%s answered, waiting for '%c' to acknowledge\n", p->chan->name, p->acceptdtmf);
+					/* Don't pass answer along */
+					ast_frfree(f);
+					f = &ast_null_frame;
+				} else {
+					p->acknowledged = 1;
+					/* Use the builtin answer frame for the 
 					   recording start check below. */
- 					ast_frfree(f);
- 					f = &answer_frame;
- 				}
- 			}
- 			break;
+					ast_frfree(f);
+					f = &answer_frame;
+				}
+			}
+			break;
 		case AST_FRAME_DTMF_BEGIN:
 			/*ignore DTMF begin's as it can cause issues with queue announce files*/
 			if((!p->acknowledged && f->subclass == p->acceptdtmf) || (f->subclass == p->enddtmf && endcall)){
